package com.roobo.svui_plugin

import android.content.Context
import android.content.pm.PackageManager
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.util.Log
import androidx.annotation.NonNull
import com.google.gson.Gson
import com.roobo.svui.SVuiSdk
import com.roobo.svui.SdkAccountInfo
import com.roobo.svui.SdkConstant
import com.roobo.svui.audio.IAudioSource
import com.roobo.svui.engine.EngineType
import com.roobo.svui.engine.cloud.AIConfigParams
import com.roobo.svui.error.ErrorCode
import com.roobo.svui.error.SVUIError
import com.roobo.svui.event.SVuiCloudCallBack
import com.roobo.svui.event.SVuiEventInfo
import com.roobo.svui.event.SVuiEventListener
import com.roobo.svui_plugin.bean.EvaluateResultBean
import com.roobo.svui_plugin.bean.SimpleResultBean
import com.roobo.svui_plugin.bean.TTSResultBean
import com.roobo.svui_plugin.utils.FileRecorder
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.io.File
import java.security.Permission
import java.security.Permissions
import java.util.*

/** SvuiPlugin */
public class SvuiPlugin : FlutterPlugin, MethodCallHandler {
    private lateinit var context: Context
    private val TAG = "MyMethodHandler";
    private var audioPath: String? = null

    //    private val confPath = Environment.getExternalStorageDirectory().absolutePath +
//            "/roobo/svui/config.svui.json"
//    private val vuiSdPath = Environment.getExternalStorageDirectory().absolutePath +
//            "/roobo/svui"
    private val confPath = "config.svui.json"
    private val vuiSdPath = "/"
    private val mSceneList = ArrayList<String>()
    private var mAudioSource: IAudioSource? = null
    private val gson = Gson()
    private var isASRInitialized: Boolean = false
    private val mH = Handler(Looper.getMainLooper())

    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        context = flutterPluginBinding.applicationContext;
        audioPath = context.getExternalFilesDir(null)?.absolutePath + File.separator
        Log.d(TAG, audioPath)
        channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "svui_plugin")
        channel.setMethodCallHandler(this);
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "svui_plugin")
            channel.setMethodCallHandler(SvuiPlugin())
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "getPlatformVersion" -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            "initSVUI" -> {
                val accountInfo = gson.fromJson(call.argument<String>("accountInfo"), SdkAccountInfo::class.java)
                var isTest = false;
                if (null != call.argument<Boolean>("isTest")) {
                    isTest = call.argument<Boolean>("isTest")!!
                }
                initSVUI(result, accountInfo, isTest)
            }
            "oralEvaluate" -> {
                oralEvaluate(result, call.argument<String>("text"))
            }
            "tts" -> {
                tts(result, call.argument<String>("text"), call.argument<String>("language"))
            }
            "stopCapture" -> {
                stopCapture(result)
            }
            "release" -> {
                release()
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    @Synchronized
    private fun initSVUI(result: MethodChannel.Result, accountInfo: SdkAccountInfo, isTest: Boolean) {
        if (isASRInitialized) {
            Log.d(TAG, "already initialized");
            mH.post(Runnable {
                result.success(true)
            })
            return
        }
        Thread(Runnable { // sdk 基本接口初始化
            SVuiSdk.getInstance().context = this.context
            SVuiSdk.getInstance().init(vuiSdPath)
//            GlobalConfig.SAVE_CLOUD_ASR_AUDIO = true
//            GlobalConfig.ASR_AUDIO_SAVE_DIR = audioPath
            SVuiSdk.getInstance().setCloudAudioSaveDir(audioPath);
            SVuiSdk.getInstance().setLogLevel(SdkConstant.LogLevel.LOG_LEVEL_VERBOSE)
            if (isTest) {
                SVuiSdk.getInstance().setSdkEnv(SdkConstant.WorkEnv.ENV_TEST)
            } else {
                SVuiSdk.getInstance().setSdkEnv(SdkConstant.WorkEnv.ENV_ONLINE)
            }
            val ret = SVuiSdk.getInstance().setAccountInfo(accountInfo)
            Log.i(TAG, "setAccountInfo $ret")
            if (ret != ErrorCode.ERROR_OK) {
                Log.d(TAG, "init failed err:$ret")
                mH.post(Runnable {
                    result.error(ret.toString(), "init failed", "init failed")
                })
                return@Runnable
            }
            SVuiSdk.getInstance().reset(SVuiSdk.RESET_FULL)
            // 加载配置文件
            val loadResult = SVuiSdk.getInstance().loadConfig(confPath)
            if (loadResult != ErrorCode.ERROR_OK) {
                Log.d(TAG, "loadConfig failed err:$loadResult")
                mH.post(Runnable {
                    result.error(loadResult.toString(), "init failed", "loadConfig failed")
                })
                return@Runnable
            }
            mSceneList.clear()
            for (scene in SVuiSdk.getInstance().sceneList) {
                Log.d(TAG, "scene:$scene")
                mSceneList.add(scene)
            }
            isASRInitialized = true
            mH.post(Runnable {
                result.success(true)
            })
        }).start()
    }

    private fun oralEvaluate(result: MethodChannel.Result, text: String?) {
        Log.d(TAG, "oralEvaluate: $text")
        if (TextUtils.isEmpty(text)) {
            return
        }
        val packageManager: PackageManager = context.packageManager
        val packageName = context.packageName
        if (PackageManager.PERMISSION_DENIED == packageManager.checkPermission(
                        "android.permission.RECORD_AUDIO", packageName)) {
            mH.post(Runnable {
                result.error("permission denied", "record_audio permission denied",
                        "record_audio permission denied")
            })
            return
        }
        SVuiSdk.getInstance().startScene("oraltest")
        SVuiSdk.getInstance().setEngineParam(EngineType.CLOUD, AIConfigParams.AI_CONFIG_ORALTESTING_EX.name,
                "{\"text\": \"$text\"}")
        SVuiSdk.getInstance().registerSVuiEventListener("oraltest", object : SVuiEventListener {
            override fun onEvent(i: Int, info: SVuiEventInfo) {
                Log.d(TAG, "onEvent eventType:$i ,info$info")
                val evaResult = gson.fromJson(info.text, EvaluateResultBean::class.java)
                val pcmPath = "${audioPath + evaResult.traceId}.pcm"
                val wavPath = "${audioPath + evaResult.traceId}.wav"
                FileRecorder.savePcmToWav(File(pcmPath), File(wavPath))
                FileRecorder.deleteFile(File(pcmPath))
                val simpleResult = SimpleResultBean(evaResult.Data?.pronunciation,
                        evaResult.Data?.words?.get(0)?.word,
                        wavPath)
                val simpleS = gson.toJson(simpleResult)
                mAudioSource?.stopCapture()
                mH.post(Runnable {
                    result.success(simpleS)
                })
            }

            override fun onSpeechEvent(i: Int) {
                Log.d(TAG, "onSpeechEvent eventType:$i")
            }

            override fun onCloudRecognizeStart(p0: String?) {
                Log.d(TAG, "save file name:$p0")
            }

            override fun onError(error: SVUIError) {
                Log.d(TAG, "onError error:$error")
                mAudioSource?.stopCapture()
                mH.post(Runnable { result.error(error.detail, error.detail, error.detail) })
            }
        })
        // 音频采集设置
        if (mAudioSource == null) {
            mAudioSource = SVuiSdk.getInstance().createAudioSource(SVuiSdk.AUDIO_SOURCE_ANDROID_DEFAULT)
        }
        if (!mAudioSource?.isCaptureStarted!!) {
            mAudioSource?.setAudioDataSink(SVuiSdk.getInstance().audioDataSink)
            val startSuccess = mAudioSource?.startCapture()
            if (!startSuccess!!) {
                mH.post(Runnable {
                    result.error("start capture failed",
                            "start capture failed", "start capture failed")
                })
            }
        }
    }

    private fun stopCapture(result: MethodChannel.Result) {
        if (null == mAudioSource) {
            result.error("audio source is null", "audio source is null", "audio source is null")
            return
        }
        mAudioSource?.stopCapture();
    }

    private fun tts(result: MethodChannel.Result, text: String?, language: String?) {
        Log.d(TAG, "tts:$text")
        SVuiSdk.getInstance().setSVuiCloudCallBack(object : SVuiCloudCallBack {
            override fun onCloudResult(p0: Int, p1: String?) {
                mH.post(Runnable {
                    val ttsResult = gson.fromJson(p1, TTSResultBean::class.java)
                    if (p0 == ErrorCode.ERROR_OK) {
                        result.success(ttsResult?.tts?.content)
                    } else {
                        result.error(p0.toString(), ttsResult?.status?.errorType, ttsResult?.status?.errorType)
                    }
                })
            }
        })
        Thread(Runnable {
            if (TextUtils.isEmpty(language)) {
                SVuiSdk.getInstance().ttsTransform(text)
            } else {
                SVuiSdk.getInstance().ttsTransform(text, language)
            }
        }).start()
    }

    private fun release() {
        //TODO release svuisdk
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}
